@@ -2,7 +2,7 @@ package Ace::Object;
 use strict;
 use Carp;
 
-# $Id: Object.pm,v 1.31 2001/01/10 05:42:23 lstein Exp $
+# $Id: Object.pm,v 1.36 2001/05/31 18:15:23 lstein Exp $
 
 use overload 
     '""'       => 'name',
@@ -39,7 +39,7 @@ sub AUTOLOAD {
 
     if ($presumed_tag) {
       croak "Invalid object tag \"$func_name\"" 
-	if $self->db && !$self->model->valid_tag($func_name);
+	if $self->db && $self->model && !$self->model->valid_tag($func_name);
 
       $self = $self->fetch if !$self->isRoot && $self->db;  # dereference, if need be
       croak "Null object tag \"$func_name\"" unless $self;
@@ -51,11 +51,11 @@ sub AUTOLOAD {
 	shift();
       }
       return $self->search($func_name,@_) if wantarray;
-      my $obj = @_ ? $self->search($func_name,@_) : $self->search($func_name,1);
+      my ($obj) = @_ ? $self->search($func_name,@_) : $self->search($func_name,1);
 
       # these nasty heuristics simulate aql semantics.
       # undefined return
-      return unless $obj;
+      return unless defined $obj;
 
       # don't dereference object if '@' symbol specified
       return $obj if $no_dereference;
@@ -66,10 +66,12 @@ sub AUTOLOAD {
       # otherwise dereference if the current thing is an object or we are at a tag
       # and the thing to the right is an object.
 #      return $obj->fetch if $obj->isObject || ($obj->right && $obj->right->isObject);  # this heuristic stinks
-      return $obj->fetch if $obj->isObject;  # always dereference objects
+      return $obj->fetch if $obj->isObject && !$obj->isRoot;  # always dereference objects
 
       # otherwise return the thing itself
       return $obj;
+    } elsif ($func_name =~ /^[A-Z]/ && $self->isTag) {  # follow tag
+      return $self->search($func_name);
     } else {
       $AutoLoader::AUTOLOAD = __PACKAGE__ . "::$func_name";
       goto &AutoLoader::AUTOLOAD;
@@ -232,12 +234,14 @@ sub col {
 
 #### Search for a tag, and return the column ####
 #### Uses a breadth-first search (cols then rows) ####
-# This is in the midst of an optimization and is more complex
-# than it should be.  It should take better advantage of the path()
-# method in Ace::Model.
 sub search {
-    my ($self,$tag,$subtag,$pos) = @_;
-    my $lctag = lc $tag;
+  my $self = shift;
+  my $tag = shift unless $_[0]=~/^-/;
+  my ($subtag,$pos,$filled) = rearrange(['SUBTAG','POS',['FILL','FILLED']],@_);
+  my $lctag = lc $tag;
+
+  # filled implies a follow
+  return $self->follow(-tag=>$tag,-filled=>$filled) if $filled;
 
     TRY: {
 
@@ -407,8 +411,11 @@ sub follow {
     return $self->fetch() unless $tag;
     my $class = $self->class;
     my $name = Ace->freeprotect($self->name);
-    return $self->db->fetch(-query=>"find $class $name ; follow $tag",
-			    '-filled'=>$filled);
+    my @options;
+    if ($filled) {
+      @options = $filled =~ /^[a-zA-Z]/ ? ('filltag' => $filled) : ('filled'=>1);
+    }
+    return $self->db->fetch(-query=>"find $class $name ; follow $tag",@options);
 }
 
 # returns true if the object has a Model, i.e, can be followed into
@@ -483,7 +490,7 @@ sub _parse {
   my $db = $self->{'db'};
 
   for (my $r=$current_row+1; $r<=$self->{'.end_row'}; $r++) {
-    next unless $raw->[$r][$col];
+    next unless $raw->[$r][$col] ne '';
 
     my $obj_right = $self->_fromRaw($raw,$current_row,$col+1,$r-1,$db);
 
@@ -494,8 +501,8 @@ sub _parse {
       while ($obj_right->isComment) {
 	$current_obj->comment($obj_right)   if $obj_right->isComment;
 	$t = $obj_right;
-	last unless $obj_right = $self->_fromRaw($raw,$row++,$col+1,$r-1,$db);
-      } 
+	last unless defined ($obj_right = $self->_fromRaw($raw,$row++,$col+1,$r-1,$db));
+      }
     }
     $current_obj->{'.right'} = $obj_right;
 
@@ -528,7 +535,7 @@ sub _fromRaw {
 #  $pack = $pack->factory();
 
   my ($raw,$start_row,$col,$end_row,$db) = @_;
-  return unless $raw->[$start_row][$col];
+  return unless defined $raw->[$start_row][$col];
   my ($class,$name,$ts) = Ace->split($raw->[$start_row][$col]);
   my $self = $pack->new($class,$name,$db,!($start_row || $col));
   @{$self}{qw(.raw .start_row .end_row .col db)} = ($raw,$start_row,$end_row,$col,$db);
@@ -583,6 +590,7 @@ sub _split_tags {
   return map { (my $x=$_)=~s/$;/./g; $x } split(/\./,$tag);
 }
 
+
 1;
 
 __END__
@@ -1012,6 +1020,7 @@ This is a deprecated synonym for get().
      @array  = $object->Name_of_tag;
      @array  = $object->Name_of_tag($position);
      @array  = $object->Name_of_tag($subtag=>$position);
+     @array  = $object->Name_of_tag(-fill=>$tag);
 
 The module attempts to autogenerate data access methods as needed.
 For example, if you refer to a method named "Fax" (which doesn't
@@ -1061,11 +1070,30 @@ Both named tags and positions can be combined as follows:
 
      $lab_address = $object->Laboratory(Address=>2);
 
-** Note that the semantics of the autogenerated methods have changed
-subtly between version 1.57 (the last stable release) and version 1.62.   
-In earlier versions, calling an autogenerated method in a scalar context
-returned the subtree rooted at the tag.  In the current version, an
-implicit right() and dereference is performed.
+If you provide a -fill=>$tag argument, then the object fetch will
+automatically fill the specified subtree, greatly improving
+performance.  For example:
+
+      $lab_address = $object->Laboratory(-filled=>'Address');
+
+** NOTE: In a scalar context, if the node to the right of the tag is
+** an object, the method will perform an implicit dereference of the
+** object.  For example, in the case of:
+
+    $lab = $author->Laboratory;
+
+**NOTE: The object returned is the dereferenced Laboratory object, not
+a node in the Author object.  You can control this by giving the
+autogenerated method a numeric offset, such as Laboratory(0) or
+Laboratory(1).  For backwards compatibility, Laboratory('@') is
+equivalent to Laboratory(1).
+
+The semantics of the autogenerated methods have changed subtly between
+version 1.57 (the last stable release) and version 1.62.  In earlier
+versions, calling an autogenerated method in a scalar context returned
+the subtree rooted at the tag.  In the current version, an implicit
+right() and dereference is performed.
+
 
 =head2 fetch() method
 
@@ -1087,6 +1115,7 @@ argument allows you to combine the two operations into a single one:
 
     @papers        = $object->follow('Paper');
     @filled_papers = $object->follow(-tag=>'Paper',-filled=>1);
+    @filled_papers = $object->follow(-tag=>'Paper',-filled=>'Author');
 
 The follow() method will follow a tag into the database, dereferencing
 the column to its right and returning the objects resulting from this
@@ -1099,10 +1128,13 @@ followed.
 
 The full named-argument form of this call accepts the arguments
 B<-tag> (mandatory) and B<-filled> (optional).  The former points to
-the tag to follow.  The latter accepts a boolean argument.  A true
-argument will return "filled" objects, increasing network and memory
-usage, but possibly boosting performance if you have a high database
-access latency.
+the tag to follow.  The latter accepts a boolean argument or the name
+of a subtag.  A numeric true argument will return completely "filled"
+objects, increasing network and memory usage, but possibly boosting
+performance if you have a high database access latency.
+Alternatively, you may provide the name of a tag to follow, in which
+case just the named portion of the subtree in the followed objects
+will be filled (v.g.)
 
 For backward compatability, if follow() is called without any
 arguments, it will act like fetch().
@@ -1345,6 +1377,8 @@ be detected and reported at commit() time however.
 
 The add() method is an alias for add_row().
 
+See also the Ace->new() method.
+
 =head2 add_tree()
 
   $result_code = $object->add_tree($tag=>$ace_object);
@@ -1744,34 +1778,6 @@ sub tags {
     return @tags;
 }
 
-################# delete a portion of the tree #############
-# Only changes local copy until you perform commit() #
-#  returns true if this is a valid thing to do.
-sub delete {
-  my $self = shift;
-  my($tag,$oldvalue,@rest) = rearrange([['TAG','PATH'],['VALUE','OLDVALUE','OLD']],@_);
-  
-  # flatten array refs into array
-  my @values;
-  @values = map { ref($_) && ref($_) eq 'ARRAY' ? @$_ : $_ } ($oldvalue,@rest) 
-    if defined($oldvalue);
-  my $row = join(".",($tag,map { (my $x = $_) =~s/\./\\./g; $x } @values));
-  my $subtree = $self->at($row,undef,1);  # returns the parent
-
-  if (@values
-      && defined($subtree->{'.right'})
-      && "$subtree->{'.right'}" eq $oldvalue) {
-    $subtree->{'.right'} = $subtree->{'.right'}->down;
-  } else {
-    $subtree->{'.down'} = $subtree->{'.down'}->{'.down'}
-  }
-
-  push(@{$self->{'.update'}},join(' ','-D',
-				 map { Ace->freeprotect($_) } ($self->_split_tags($tag),@values)));
-  delete $self->{'.PATHS'}; # uncache cached values
-  1;
-}
-
 ################# kill an object ################
 # Removes the object from the database immediately.
 sub kill {
@@ -1812,9 +1818,14 @@ sub add_row {
   my @values = map { ref($_) && ref($_) eq 'ARRAY' ? @$_ : $_ } @newvalue;
 
   # make sure that this entry doesn't already exist
+  unless ($tag =~ /\./) {
+    my $model = $self->model;
+    my @intermediate_tags = $model->path($tag);
+    $tag = join '.',@intermediate_tags,$tag;
+  }
   my $row = join(".",($tag,map { (my $x = $_) =~s/\./\\./g; $x } @values));
   return if $self->at($row);  # an identical row already exists in the object
-  
+
   # If we get here then we need to turn @values into an array of Ace::Objects
   # for insertion.  Also need to link them together into a row.
   my $previous;
@@ -1858,6 +1869,12 @@ sub add_tree {
   my($tag,$value,@rest) = rearrange([['TAG','PATH'],['VALUE','TREE']],@_);
   croak "Value must be an Ace::Object" unless ref($value) && $value->isa('Ace::Object');
 
+  unless ($tag =~ /\./) {
+    my $model = $self->model;
+    my @intermediate_tags = $model->path($tag);
+    $tag = join '.',@intermediate_tags,$tag;
+  }
+
   # position at the indicated tag, creating it if necessary
   my (@tags) = $self->_split_tags($tag);
   my $p = $self;
@@ -1880,6 +1897,42 @@ sub add_tree {
   1;
 }
 
+################# delete a portion of the tree #############
+# Only changes local copy until you perform commit() #
+#  returns true if this is a valid thing to do.
+sub delete {
+  my $self = shift;
+  my($tag,$oldvalue,@rest) = rearrange([['TAG','PATH'],['VALUE','OLDVALUE','OLD']],@_);
+
+  # flatten array refs into array
+  my @values;
+  @values = map { ref($_) && ref($_) eq 'ARRAY' ? @$_ : $_ } ($oldvalue,@rest) 
+    if defined($oldvalue);
+
+  unless ($tag =~ /\./) {
+    my $model = $self->model;
+    my @intermediate_tags = $model->path($tag);
+    $tag = join '.',@intermediate_tags,$tag;
+  }
+
+  my $row = join(".",($tag,map { (my $x = $_) =~s/\./\\./g; $x } @values));
+  my $subtree = $self->at($row,undef,1);  # returns the parent
+
+  if (@values
+      && defined($subtree->{'.right'})
+      && "$subtree->{'.right'}" eq $oldvalue) {
+    $subtree->{'.right'} = $subtree->{'.right'}->down;
+  } else {
+    $subtree->{'.down'} = $subtree->{'.down'}->{'.down'}
+  }
+
+  push(@{$self->{'.update'}},join(' ','-D',
+				 map { Ace->freeprotect($_) } ($self->_split_tags($tag),@values)));
+  delete $self->{'.PATHS'}; # uncache cached values
+  1;
+}
+
+
 ################# delete a portion of the tree #############
 # Only changes local copy until you perform commit() #
 #  returns true if this is a valid thing to do #
@@ -1908,7 +1961,7 @@ sub commit {
 
   $Ace::Error = '';
   my $result = '';
-  
+
   # bad design alert: the following breaks encapsulation
   if ($db->{database}->can('write')) { # new way for socket server
     my $cmd = join "\n","$self->{'class'} : $name",@{$self->{'.update'}};
@@ -1926,8 +1979,13 @@ sub commit {
   } elsif (defined($result) and $result =~ /sorry|parse error/mi) {
     $Ace::Error = $result;
   }
+  return if $Ace::Error;
   undef $self->{'.update'};
-  return !$Ace::Error;
+  # this will force a fresh retrieval of the object
+  # and synchronize our in-memory copy with the db
+  delete $self->{'.right'};
+  delete $self->{'.PATHS'};
+  return 1;
 }
 
 # undo changes
@@ -1937,6 +1995,7 @@ sub rollback {
     # this will force object to be reloaded from database
     # next time it is needed.
     delete $self->{'.right'};
+    delete $self->{'.PATHS'};
     1;
 }
 
